@@ -17,18 +17,30 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 #include "playfield.h"
+#include "xml-util.h"
 
 #include <string.h>
 
 Tile *get_tile (PlayField *pf, gint row, gint col);
-void set_tile (PlayField *pf, gint row, gint col, Tile *tile);
-static void read_tile (PlayField *pf, guint row, guint col, xmlNodePtr node);
+void set_tile (PlayField *pf, guint row, guint col, Tile *tile);
 
 static GObjectClass *parent_class = NULL;
 
 static void playfield_class_init (GObjectClass *class);
 static void playfield_init (PlayField *pf);
 static void playfield_finalize (GObject *object);
+static void
+position_parser_start_element (GMarkupParseContext  *context,
+                               const gchar          *element_name,
+                               const gchar         **attribute_names,
+                               const gchar         **attribute_values,
+                               gpointer              user_data,
+                               GError              **error);
+static void
+position_parser_end_element (GMarkupParseContext  *context,
+                             const gchar          *element_name,
+                             gpointer              user_data,
+                             GError              **error);
 
 struct _PlayFieldPrivate
 {
@@ -37,6 +49,13 @@ struct _PlayFieldPrivate
   Tile **matrix;
 };
 
+typedef struct
+{
+  guint row;
+  guint col;
+  Tile *tile;
+} TileData;
+
 GType playfield_get_type (void)
 {
   static GType object_type = 0;
@@ -83,7 +102,7 @@ static void playfield_init (PlayField *pf)
 
 static void playfield_finalize (GObject *object)
 {
-  gint row, col;
+  guint row, col;
   PlayField *pf = PLAYFIELD (object);
 
 #ifdef DEBUG
@@ -123,13 +142,15 @@ Tile *get_tile (PlayField *pf, gint row, gint col)
   return priv->matrix[row * priv->n_cols + col];
 }
 
-void set_tile (PlayField *pf, gint row, gint col, Tile *new_tile)
+void set_tile (PlayField *pf, guint row, guint col, Tile *new_tile)
 {
   Tile *tile;
   PlayFieldPrivate *priv;
 
   g_return_if_fail (IS_PLAYFIELD (pf));
 
+  g_return_if_fail (row < pf->priv->n_rows || col < pf->priv->n_cols);
+
   priv = pf->priv;
 
   tile = get_tile (pf, row, col);
@@ -197,8 +218,8 @@ void playfield_add_column (PlayField *pf)
 
 void playfield_set_matrix_size (PlayField *pf, guint n_rows, guint n_cols)
 {
-  gint row = 0;
-  gint col = 0;
+  guint row = 0;
+  guint col = 0;
   guint old_n_rows;
   guint old_n_cols;
   Tile **new_matrix;
@@ -218,21 +239,21 @@ void playfield_set_matrix_size (PlayField *pf, guint n_rows, guint n_cols)
     {
       // free the left over tiles;
       for (row = 0; row < old_n_rows; row++)
-	{
-	  for (col = 0; col < old_n_cols; col++)
-	    {
-	      Tile *tile = get_tile (pf, row, col);
-	      if (row >= n_rows && tile)
-		g_object_unref (tile);
-	      else if (col >= n_cols && tile)
-		g_object_unref (tile);
-	    }
-	}
+      {
+        for (col = 0; col < old_n_cols; col++)
+        {
+          Tile *tile = get_tile (pf, row, col);
+          if (row >= n_rows && tile)
+            g_object_unref (tile);
+          else if (col >= n_cols && tile)
+            g_object_unref (tile);
+        }
+      }
     }
 
   new_matrix = g_malloc0 (n_rows * n_cols * sizeof (Tile *));
   memcpy (new_matrix, pf->priv->matrix,
-	  old_n_rows * old_n_cols * sizeof (Tile *));
+      old_n_rows * old_n_cols * sizeof (Tile *));
 
   g_free (pf->priv->matrix);
   pf->priv->matrix = new_matrix;
@@ -243,7 +264,7 @@ void playfield_set_matrix_size (PlayField *pf, guint n_rows, guint n_cols)
 PlayField *playfield_copy (PlayField *pf)
 {
   PlayField *pf_copy = NULL;
-  gint row, col;
+  guint row, col;
   gint matrix_size;
 
   g_return_val_if_fail (IS_PLAYFIELD (pf), NULL);
@@ -268,7 +289,7 @@ void playfield_swap_tiles (PlayField *pf, guint src_row, guint src_col,
 {
   Tile *src_tile;
   Tile *dest_tile;
-  gint n_rows, n_cols;
+  guint n_rows, n_cols;
 
   g_return_if_fail (IS_PLAYFIELD (pf));
 
@@ -315,7 +336,7 @@ Tile *playfield_clear_tile (PlayField *pf, guint row, guint col)
 
 void playfield_clear (PlayField *pf)
 {
-  gint row, col;
+  guint row, col;
 
   g_return_if_fail (IS_PLAYFIELD (pf));
 
@@ -327,12 +348,12 @@ void playfield_clear (PlayField *pf)
 PlayField *playfield_strip (PlayField *pf)
 {
   PlayField *stripped_pf;
-  gint row, col;
-  gint n_rows, n_cols;
-  gint max_row = 0;
-  gint min_row = 10000;
-  gint max_col = 0;
-  gint min_col = 10000;
+  guint row, col;
+  guint n_rows, n_cols;
+  guint max_row = 0;
+  guint min_row = 10000;
+  guint max_col = 0;
+  guint min_col = 10000;
 
   g_return_val_if_fail (IS_PLAYFIELD (pf), NULL);
 
@@ -403,7 +424,7 @@ void playfield_set_tile (PlayField *pf, guint row, guint col, Tile *tile)
 void playfield_print (PlayField *pf)
 {
   PlayFieldPrivate *priv;
-  int row, col;
+  guint row, col;
   Tile *tile;
 
   g_return_if_fail (IS_PLAYFIELD (pf));
@@ -428,78 +449,6 @@ void playfield_print (PlayField *pf)
   g_print ("\n");
 }
 
-PlayField *playfield_new_from_xml (xmlNodePtr node)
-{
-  xmlNodePtr child_node;
-  PlayField *pf;
-  gint row, col;
-  gint n_rows, n_cols;
-  gchar *prop_value;
-  gchar *content;
-
-  g_return_val_if_fail (node != NULL, NULL);
-
-  pf = playfield_new ();
-  row = 0;
-  col = 0;
-  n_rows = 0;
-  n_cols = 0;
-
-  /* reading non empty tiles */
-  for (; node != NULL; node = node->next)
-    {
-      if (!g_ascii_strcasecmp (node->name, "n_rows"))
-	{
-	  content = xmlNodeGetContent (node);
-	  n_rows = atoi (content);
-	}
-      else if (!g_ascii_strcasecmp (node->name, "n_columns"))
-	{
-	  content = xmlNodeGetContent (node);
-	  n_cols = atoi (content);
-	  playfield_set_matrix_size (pf, n_rows, n_cols);
-	}
-      else if (!g_ascii_strcasecmp (node->name, "row"))
-	{
-	  prop_value = xmlGetProp (node, "no");
-	  row = atoi (prop_value);
-	  for (child_node = node->xmlChildrenNode;
-	       child_node != NULL; child_node = child_node->next)
-	    {
-	      if (!g_ascii_strcasecmp (child_node->name, "col"))
-		{
-		  prop_value = xmlGetProp (child_node, "no");
-		  col = atoi (prop_value);
-		  read_tile (pf, row, col, child_node->xmlChildrenNode);
-		}
-	    }
-	}
-      else if (!g_ascii_strcasecmp (node->name, "text"))
-	{
-	}
-      else
-	{
-	  g_warning ("Skipping unexpected Tag %s.", node->name);
-	}
-    }
-  return pf;
-}
-
-static void read_tile (PlayField *pf, guint row, guint col, xmlNodePtr node)
-{
-  Tile *tile;
-
-  for (; node != NULL; node = node->next)
-    {
-      if (!g_ascii_strcasecmp (node->name, "tile"))
-	{
-	  tile = tile_new_from_xml (node);
-	  playfield_set_tile (pf, row, col, tile);
-	  g_object_unref (tile);
-	}
-    }
-}
-
 /*=================================================================
 
   Functions for generating playfields from basic level descriptions.
@@ -509,7 +458,7 @@ static void read_tile (PlayField *pf, guint row, guint col, xmlNodePtr node)
 typedef struct
 {
   GQuark id;
-  gchar *string;
+  const gchar *string;
 } TranslationItem;
 
 static TranslationItem wall_map[] =
@@ -572,15 +521,11 @@ static const offset env_offset[8] =
     {-1, -1}			/* ENV_TOP_LEFT */
   };
 
-static TileType get_env_tile_type (PlayField *pf, gint row, gint col)
+static TileType get_env_tile_type (PlayField *pf, guint row, guint col)
 {
   Tile *tile;
   TileType type = TILE_TYPE_NONE;
 
-  if (row < 0)
-    return type;
-  if (col < 0)
-    return type;
   if (row >= playfield_get_n_rows (pf))
     return type;
   if (col >= playfield_get_n_cols (pf))
@@ -642,14 +587,14 @@ PlayField *playfield_generate_environment (PlayField *pf, Theme *theme)
 {
   PlayField *env_pf;
   PlayFieldPrivate *priv;
-  gint row, col;
+  guint row, col;
   gint tile_env[8];
   Tile *env_tile;
   Tile *tile = NULL;
   gint n_decor_tiles;
   gint max_try;
-  gint min_wall[2];
-  gint max_wall[2];
+  guint min_wall[2];
+  guint max_wall[2];
 
   g_return_val_if_fail (IS_PLAYFIELD (pf), NULL);
 
@@ -771,7 +716,7 @@ PlayField *playfield_generate_shadow (PlayField *pf)
 {
   PlayField *env_pf;
   PlayFieldPrivate *priv;
-  gint row, col;
+  guint row, col;
   gint tile_env[8];
   Tile *env_tile;
   Tile *tile;
@@ -818,3 +763,132 @@ PlayField *playfield_generate_shadow (PlayField *pf)
 
   return env_pf;
 }
+
+/*=================================================================
+
+  Functions and structures for parsing playfields from files.
+
+ -----------------------------------------------------------------*/
+
+static GMarkupParser tile_parser =
+{
+  tile_parser_start_element,
+  tile_parser_end_element,
+  NULL,
+  NULL,
+  xml_parser_log_error
+};
+
+static GMarkupParser position_parser =
+{
+  position_parser_start_element,
+  position_parser_end_element,
+  NULL,
+  NULL,
+  xml_parser_log_error
+};
+
+static void
+position_parser_start_element (GMarkupParseContext  *context,
+                               const gchar          *element_name,
+                               const gchar         **attribute_names,
+                               const gchar         **attribute_values,
+                               gpointer              user_data,
+                               GError              **error)
+{
+  Tile *tile = NULL;
+  TileType type = TILE_TYPE_UNKNOWN;
+  gint base_id = 0;
+
+  if (!g_strcmp0 (element_name, "tile"))
+  {
+    type  = tile_type_from_string (get_attribute_value ("type", attribute_names, attribute_values));
+    tile = tile_new (type);
+    base_id = g_quark_from_string (get_attribute_value ("base", attribute_names, attribute_values));
+    tile_set_base_id (tile, base_id);
+    g_markup_parse_context_push (context, &tile_parser, tile);
+  } else
+  {
+    printf ("position: starting %s\n", element_name);
+  }
+}
+
+static void
+position_parser_end_element (GMarkupParseContext  *context,
+                             const gchar          *element_name,
+                             gpointer              user_data,
+                             GError              **error)
+{
+  TileData *tile_data = user_data;
+
+
+  if (!g_strcmp0 (element_name, "tile"))
+  {
+    tile_data->tile = g_markup_parse_context_pop (context);
+  } else
+  {
+    printf ("position: ending %s\n", element_name);
+  }
+}
+
+void
+playfield_parser_start_element (GMarkupParseContext  *context,
+                                const gchar          *element_name,
+                                const gchar         **attribute_names,
+                                const gchar         **attribute_values,
+                                gpointer              user_data,
+                                GError              **error)
+{
+  TileData *tile_data = NULL;
+  const gchar *current = NULL;
+  gint number = 0;
+
+  if (!g_strcmp0 (element_name, "position"))
+  {
+    tile_data = g_slice_new (TileData);
+
+    current = get_attribute_value ("row", attribute_names, attribute_values);
+    number = atoi (current);
+    tile_data->row = (guint) number;
+
+    current = get_attribute_value ("col", attribute_names, attribute_values);
+    number = atoi (current);
+    
+    tile_data->col = (guint) number;
+    tile_data->tile = NULL;
+    g_markup_parse_context_push (context, &position_parser, tile_data);
+  } else
+  {
+    printf ("playfield: starting %s\n", element_name);
+  }
+}
+
+void
+playfield_parser_text (GMarkupParseContext  *context,
+                       const gchar          *text,
+                       gsize                 text_len,
+                       gpointer              user_data,
+                       GError              **error)
+{
+  //printf ("playfield: text %s\n", text);
+}
+
+void
+playfield_parser_end_element (GMarkupParseContext  *context,
+                              const gchar          *element_name,
+                              gpointer              user_data,
+                              GError              **error)
+{
+  PlayField *playfield = user_data;
+  TileData *tile_data = NULL;
+
+  if (!g_strcmp0 (element_name, "position"))
+  {
+    tile_data = g_markup_parse_context_pop (context);
+    playfield_set_tile (playfield, tile_data->row, tile_data->col, tile_data->tile);
+    g_slice_free (TileData, tile_data);
+  } else
+  {
+    printf ("playfield: ending %s\n", element_name);
+  }
+}
